#!/usr/bin/env python
#
# gdb.py: Python module for low level GDB protocol implementation
# Copyright (C) 2009  Black Sphere Technologies
# Written by Gareth McMullin <gareth@blacksphere.co.nz>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Used to parse XML memory map from target
from xml.dom.minidom import parseString
import struct,array
import time

def hexify(s):
    """Convert a bytes object into hex bytes representation"""
    return s.hex().encode()

def unhexify(s):
    """Convert a hex-encoded bytes into bytes object"""
    return bytes.fromhex(s.decode())

class GetPacketTimeoutException(Exception):
    pass

class InvalidStubResponseException(Exception):
    pass

class FakeSocket():
    """Emulate socket functions send and recv on a file object"""
    def __init__(self, file):
        self.file = file
        self.sendall = self.send

    def send(self, data):
        self.file.write(data)

    def gettimeout(self):
        return self.file.timeout

    def settimeout(self, seconds):
        self.file.timeout = seconds

    def recv(self, bufsize):
        return self.file.read(bufsize)

    def flushInput(self):
        #print("hanging out",)
        extra = self.file.read(self.file.inWaiting())
        #print(extra)

class FlashMemory(object):
    def __init__(self, flash_ranges):
        self.flash_ranges = flash_ranges

    class Segment:
        def __init__(self, offset, length, blocksize):
            self.offset = offset
            self.length = length
            self.blocksize = blocksize
            self.blocks = list(None for i in range(length // blocksize))

        def prog(self, offset, data):
            assert type(data)==bytes

            assert ((offset >= self.offset) and
                (offset + len(data) <= self.offset + self.length))

            while data:
                index = (offset - self.offset) // self.blocksize
                bloffset = (offset - self.offset) % self.blocksize
                bldata = data[:self.blocksize-bloffset]
                data = data[len(bldata):]; offset += len(bldata)
                if self.blocks[index] is None: # Initialize a clear block
                    self.blocks[index] = bytes(0xff for i in range(self.blocksize))
                self.blocks[index] = (self.blocks[index][:bloffset] + bldata +
                        self.blocks[index][bloffset+len(bldata):])

    def flash_probe(self):
        self.mem = []
        for offset, length, blocksize in self.flash_ranges:
            mem = FlashMemory.Segment(offset, length, blocksize)
            self.mem.append(mem)

        return self.mem

    def flash_prepare_hex(self, hexfile):
        lowest_addr = 0xffffffff
        highest_addr = 0

        f = open(hexfile)
        addrhi = 0
        addr16 = 0
        for line in f:
            if line[0] != ':': raise Exception("Error in hex file")
            reclen = int(line[1:3], 16)
            addrlo = int(line[3:7], 16)
            rectype = int(line[7:9], 16);
            if sum(unhexify(line[1:11+reclen*2])) & 0xff != 0:
                raise Exception("Checksum error in hex file")
            if rectype == 0: # Data record
                addr = (addrhi << 16) + (addr16 << 4) + addrlo
                data = unhexify(line[9:9+reclen*2])
                if addr < lowest_addr: lowest_addr = addr
                if addr+len(data) > highest_addr: highest_addr = addr+len(data)
                self.flash_write_prepare(addr, data)
                pass
            elif rectype == 4: # High address record
                addrhi = int(line[9:13], 16)
                pass
            elif rectype == 2: # Extend address record
                addr16 = int(line[9:13], 16)
                pass
            elif rectype == 5: # Entry record
                pass
            elif rectype == 1: # End of file record
                break
            elif rectype == 3: # Start segment address...
                break
            else:
                raise Exception("Invalid record in hex file")

        return lowest_addr, highest_addr

    def flash_write_prepare(self, address, data):
        for m in self.mem:
            if (address >= m.offset) and (address + len(data) <= m.offset + m.length):
                 m.prog(address, data)

    @property
    def ihex(self):
        """Returns the same data in a intelhex-derived object.

        Documentation at http://pythonhosted.org/IntelHex/

        It would be nice to unify the code, but until then this
        function provides some glue."""

        import intelhex
        ret = intelhex.IntelHex()

        for segment in self.mem:
            data = []
            for num,block in enumerate(segment.blocks):
                if block:
                    for i,c in enumerate(block):
                        address = (segment.offset +
                                   num * segment.blocksize +
                                   i)
                        ret[address] = c
        return ret

class Target(FlashMemory):
    def __init__(self, sock):
        if "send" in dir(sock):
            self.sock = sock

            # Debugging involves back-and-forth with lots of tiny packets
            # Nagle's algorithm makes debug 10x slower
            import socket
            self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)

        else:
            self.sock = FakeSocket(sock)

        self.PacketSize=0x100 # default
        self.sock.send(b'+')
        self.sock.flushInput()
        self.get_supported()

    def getpacket(self, timeout=3, skipdollar=False):
        """Return the first correctly received packet from GDB target"""

        old_timeout = self.sock.gettimeout()
        self.sock.settimeout(timeout)
        try:
            return self.getpacket_(timeout, skipdollar)
        finally:
            self.sock.settimeout(old_timeout)

    def getpacket_(self, timeout, skipdollar):
        t0 = time.time()
        while True:
            while True and not skipdollar:
                c = self.sock.recv(1)
                if c == b'$':
                    break
                if time.time() - t0 >  timeout:
                    raise GetPacketTimeoutException()

            csum = 0
            packet = [] # list-of-small-int

            while True:
                c, = self.sock.recv(1)
                if c == ord('#'): break

                if c == ord('$'):
                    packet = []
                    csum = 0
                    continue

                if c == ord('}'):
                    c, = self.sock.recv(1)
                    csum += c + ord('}')
                    packet.append(c ^ 0x20)
                    continue

                packet.append(c)
                csum += c

            if (csum & 0xFF) == int(self.sock.recv(2),16):
                break

            self.sock.send(b'-')

        self.sock.send(b'+')
        return bytes(packet)

    def _singlepacket(self, packet):
        out = []

        for c in packet:
            if (c in b'$#}'):
                out.append(ord('}'))
                out.append(c ^ 0x20)
            else:
                out.append(c)

        csum = sum(out)
        outb = b'$'+bytes(out)+b'#%02X' % (csum & 0xff)

        self.sock.send(outb),outb

    def putpacket(self, packet):
        """Send packet to GDB target and wait for acknowledge
        packet is bytes or string"""

        if type(packet) == str:
            packet = packet.encode()

        while True:
            self._singlepacket(packet)

            c = self.sock.recv(1)
            if c == b'+':
                break
            if c == b'$':
                print("Skipped packet-instead-of-ack")
                pack=self.getpacket(skipdollar=True)
                print(pack)
                raise Exception

    def get_supported(self):
        self.putpacket(b"qSupported")
        supported = self.getpacket()
        self.supported_features=set()
        self.unsupported_features=set()

        for feature in supported.split(b";"):
            if feature.endswith(b'+'):
                self.supported_features.add(feature)
            elif feature.endswith(b'-'):
                self.unsupported_features.add(feature)
            elif b'=' in feature:
                key,val = feature.split(b'=',1)
                setattr(self,key.decode(),int(val,16))

    def monitor(self, cmd):
        """Send gdb "monitor" command to target"""
        if type(cmd) == str:
            cmd = cmd.encode()

        ret = []
        self.putpacket(b"qRcmd," + hexify(cmd))

        while True:
            s = self.getpacket()

            if s == b'': return None
            if s == b'OK': return ret
            if s.startswith(b'O'):
                ret.append(unhexify(s[1:]))
            else:
                raise InvalidStubResponseException('Invalid GDB stub response %r'%s)

    def monitor_s(self, cmd):
        "like monitor() but returns string"
        return ''.join(l.decode() for l in self.monitor(cmd))

    @property
    def target_name(self):
        connected = [name for i, name, connected
                     in self.targets()
                     if connected]
        if len(connected)==0:
            return None
        elif len(connected)==1:
            return connected[0]
        else:
            raise Exception('more than one target connected')

    def targets(self):
        "Returns list of target id, name, connected"
        ret = []
        import re
        target_re = re.compile(b'([0-9]+)(.*)')
        for line in self.monitor('targets'):
            l = line.strip()
            m = target_re.match(l)
            if m:
                index,name = m.groups()
                index = int(index)
                name = name.strip()
                if name.startswith(b'*'):
                    connected = True
                    name = name[1:]
                else:
                    connected = False

                name = name.strip().decode()
                yield (index, name, connected)


    def attach(self, pid):
        """Attach to target process (gdb "attach" command)"""
        self.putpacket(b"vAttach;%08X" % pid)
        reply = self.getpacket()
        if (reply == b'') or (reply[:1] == b'E'):
            raise Exception('Failed to attach to remote pid %d' % pid)
        self.last_stub = None

    def detach(self):
        """Detach from target process (gdb "detach" command)"""
        self.putpacket(b"D")
        if self.getpacket() != b'OK':
            raise Exception("Failed to detach from remote process")

    def reset(self):
        """Reset the target system"""
        self.putpacket(b"r")

    def read_mem(self, addr, length):
        """Read length bytes from target at address addr"""
        addr = int(addr)
        length = int(length)
        ret = b''
        while length:
            # print "Read"
            packlen = min(length,self.PacketSize//2)
            self.putpacket(b"m%08X,%08X" % (addr, packlen))
            reply = self.getpacket()
            if (reply == b'') or (reply[:1] == b'E'):
                raise Exception('Error reading memory at 0x%08X' % addr)
            try:
                data = unhexify(reply)
            except Exception:
                raise Exception('Invalid response to memory read packet: %r' % reply)
            ret += data
            length -= packlen
            addr += packlen

        return ret

    def write_mem(self, addr, data):
        """Write data to target at address addr"""
        data = bytes(data)

        while data:
            d = data[:self.PacketSize-44]
            data = data[len(d):]
            #print("Writing %d bytes at 0x%X" % (len(d), addr))
            pack = b"X%08X,%08X:%s" % (addr, len(d), d)
            self.putpacket(pack)

            response = self.getpacket()
            if response != b'OK':
                raise Exception('%s Error writing to memory at 0x%08X' % (response, addr))
            addr += len(d)

    def write32(self, address, value):
        """Convenience function.
        uint32_t little-endian values are everywhere.
        """
        assert address & 3 == 0
        #print("write %08x to %08x"%(value, address))
        self.write_mem(address, struct.pack('<I',value))

    def read32(self, address):
        assert address & 3 == 0
        value, = struct.unpack('<I',self.read_mem(address, 4))
        #print("read %08x from %08x"%(value, address))
        return value

    def read_regs(self):
        """Read target core registers"""
        self.putpacket(b"g")
        reply = self.getpacket()
        if (reply == b'') or (reply[:1] == b'E'):
            raise Exception('Error reading memory at 0x%08X' % addr)
        try:
            data = unhexify(reply)
        except Exception:
            raise Exception('Invalid response to memory read packet: %r' % reply)
        ret = array.array('I',data)
        return ret

    def write_regs(self, *regs):
        """Write target core registers"""
        data = struct.pack("=%dL" % len(regs), *regs)
        self.putpacket(b"G" + hexify(data))
        if self.getpacket() != b'OK':
            raise Exception('Error writing to target core registers')

    def memmap_read(self):
        """Read the XML memory map from target"""
        offset = 0
        ret = b''
        while True:
            self.putpacket(b"qXfer:memory-map:read::%08X,%08X" % (offset, 512))
            reply = self.getpacket()
            if (reply[0] in b'ml'):
                offset += len(reply) - 1
                ret += reply[1:]
            else:
                raise Exception('Invalid GDB stub response %r'%reply)

            if reply[:1] == b'l': return ret

    def resume(self):
        """Resume target execution"""
        self.putpacket(b"c")
        #self.last_stub = None

    def interrupt(self):
        """Interrupt target execution"""
        self.sock.send(b"\x03")
        self.last_stub = None
        self.await_stop_response('SIGINT')

    def run_stub_timeout(self, timeout, stub, address, *args):
        """Execute a binary stub at address, passing args in core registers."""
        #self.reset() # Ensure processor is in sane state
        #time.sleep(0.1)

        def pr():
            regnames = "r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 r10 r11 r12 sp lr pc xpsr fpscr msp psp special".split()
            print('\n'.join("%s = 0x%x"%(a,b) for a,b in zip(regnames,self.read_regs())))

        if not stub==self.last_stub:
            self.write_mem(address, stub)
            self.last_stub = stub

            # disable interrupts by writing ICE:
            self.write_mem(0xE000E180, b'\xff'*4*8)
            self.write_mem(0xE000E280, b'\xff'*4*8)

        stack_pointer = self.ram[0][0] + self.ram[0][1] # end of ram

        regs = list(self.read_regs())
        regs[:len(args)] = args
        old_pc = regs[15]
        regs[15] = address # pc
        regs[17] = stack_pointer # msp, sets sp
        regs[18] = regs[17] # psp, just in case
        self.write_regs(*regs)
        regs = list(self.read_regs())
        assert regs[15] == address
        self.resume()
        self.await_stop_response('SIGTRAP', timeout=timeout)

    def await_stop_response(self, await_signame, timeout=5):
        reply = None
        while not reply:
            reply = self.getpacket(timeout=timeout)

        reply_signame = {b'T02':'SIGINT',
                         b'T05':'SIGTRAP',
                         b'T0B':'SIGSEGV',
                         b'T1D':'SIGLOST'}.get(reply,repr(reply))

        if not reply_signame == await_signame:
            message = "Invalid stop response: %r" % reply
            message += ' (%s)'%reply_signame
            raise Exception(message)

    def run_stub(self, stub, address, *args):
        return self.run_stub_timeout(3, stub, address, *args)

    def flash_erase(self, startaddr, length):
        #print "Erasing flash at 0x%X" % startaddr
        self.putpacket(b"vFlashErase:%08X,%08X" %
                       (startaddr, length))
        if self.getpacket() != b'OK':
            raise Exception("Failed to erase flash")

    def commit(self, mem, progress_cb=None, erase=True):
        """Commits the blocks of memory to flash.

        Returns a tuple of (address, length, crc32), which could be
        used for verification.
        """

        totalblocks = 0
        for b in mem.blocks:
            if b is not None: totalblocks += 1

        combined = b''
        for i in range(len(mem.blocks)):
            data = mem.blocks[i]
            if data is None:
                data = b''
            # pad
            data += b'\xff' * (mem.blocksize - len(data))
            combined += data

        ret = (mem.offset, len(combined), binascii.crc32(combined))

        block = 0
        for i in range(len(mem.blocks)):
            data = mem.blocks[i]
            addr = mem.offset + mem.blocksize * i
            if data is None: continue

            block += 1
            if isinstance(progress_cb, collections.Callable):
                progress_cb(block*100//totalblocks)

            # Erase the block
            if erase:
                self.flash_erase(mem.offset + mem.blocksize*i, mem.blocksize)

            while data:
                d = data[:self.PacketSize-44]
                data = data[len(d):]
                #print "Writing %d bytes at 0x%X" % (len(d), addr)
                self.putpacket(b"vFlashWrite:%08X:%s" % (addr, d))
                addr += len(d)
                if self.getpacket() != b'OK':
                    raise Exception("Failed to write flash")

            self.putpacket(b"vFlashDone")
            if self.getpacket() != b'OK':
                raise Exception("Failed to commit")

        mem.blocks = list(None for i in range(mem.length // mem.blocksize))
        return ret

    def flash_probe(self):
        self.mem = []
        self.ram = []
        xmldom = parseString(self.memmap_read())

        for memrange in xmldom.getElementsByTagName("memory"):
            mem_type = memrange.getAttribute("type")
            if mem_type == "flash":
                offset = eval(memrange.getAttribute("start"))
                length = eval(memrange.getAttribute("length"))
                for property in memrange.getElementsByTagName("property"):
                    if property.getAttribute("name") == "blocksize":
                        blocksize = eval(property.firstChild.data)
                        break
                mem = FlashMemory.Segment(offset, length, blocksize)
                self.mem.append(mem)
            elif mem_type == "ram":
                offset = eval(memrange.getAttribute("start"))
                length = eval(memrange.getAttribute("length"))
                self.ram.append((offset, length))

            else:
                print("Unknown mem_type",mem_type)

        xmldom.unlink()

        return self.mem

    def flash_commit(self, progress_cb=None, erase=True):
        ret = []
        for m in self.mem:
            ret.append(self.commit(m, progress_cb, erase))
            print()
        return ret

    def flash_write_hex(self, hexfile, progress_cb=None, erase=False):
        self.flash_probe()
        self.flash_prepare_hex(self, hexfile)
        try:
            self.flash_commit(progress_cb, erase)
        except:
            print("Flash write failed! Is device protected?\n")
            raise



#!/usr/bin/env python3

"""Uses gcc to compile "stub" sections as used in gdb_py.Gdb.run_stub().

Prerequisites:

 - working arm-none-eabi-gcc (and objcopy) on the PATH

Writing functions:

 - To keep from reinventing an operating system, each stub is one
   function (and any functions it calls).

 - The function must be named 'function'

 - The function can take up to four 32-bit arguments.  For more
   advanced usage, put some data in RAM somewhere and pass its
   address.  This also gets data out of the function.

 - Nothing special is done with the return value in run_stub().  The
   desperate can manually decode it, looking at registers and stack
   according to the ABI. Or extend run_stub().  Or just declare void
   and pass a pointer to put the result somewhere.

 - Python's struct and array classes are useful for encoding or
   decoding inputs and outputs. ctypes looks like it would work, but
   it is set up for the processor and ABI that Python was compiled
   for.

nRF5x notes:

 - To be portable, do not explicitly include nrf5x.h or
   nrf5x_bitfields.h.  The stub generator will do this for you.

"""

import tempfile,shutil
import subprocess

CC = "arm-none-eabi-gcc"

try:
    subprocess.check_output([CC,"--version"])
except subprocess.CalledProcessError:
    raise Exception("The program '%s' is not installed." % CC)

class MakeStub():
    def __init__(self, device_svd,
                 function_addr=0x20000000, heap_addr=0x20002000):
        self.include_path=[]
        self.cflags=['-Os',
                     '--std=gnu99',
                     '-mthumb',
                     '-mabi=aapcs',
                     '-nostdlib',
                     '-Wl,-Ttext,0x%x'%function_addr,
                     '-Wl,-Tdata,0x%x'%(heap_addr-0x400),
        ]
        self.defines=[]
        self.includes=[]
        self.sources=[]

        cpu = device_svd._cpu
        if cpu: # Not all SVD contain this information
            self.cflags.append({'CM0':'-mcpu=cortex-m0',
                                'CM3':'-mcpu=cortex-m3',
                                'CM4':'-mcpu=cortex-m4'}[cpu._name])

        self.function_addr = function_addr
        self.heap_addr = heap_addr

    def __call__(self,
                 c_code,
                 extra_include_path=None,
                 extra_includes=None,
                 extra_defines=None,
                 extra_cflags=None,
                 extra_sources=None):

        if 1:
            tmpdir = tempfile.mkdtemp(prefix='svd_gdb_')
        else:
            tmpdir = "/tmp/current"

        extra_include_path = extra_include_path or []
        extra_includes = extra_includes or []
        extra_defines = extra_defines or []
        extra_cflags = extra_cflags or []
        extra_sources = extra_sources or []

        print("""
.global _start
.extern function

_start:
    bl function
    bkpt
""", file=open(tmpdir+'/call.S','w'))

        c_code = '\n'.join(['#include '+i for i in self.includes + extra_includes] + [c_code])

        print(c_code, file=open(tmpdir+'/function.c','w'))

        cc_args = ([CC] +
                   self.cflags + extra_cflags +
                   ['-D%s'%d for d in self.defines+extra_defines] +
                   ['-I%s'%i for i in self.include_path+extra_include_path] +
                   ['-g3'] +
                   ['-o',tmpdir+'/a.out'] +
                   [tmpdir+'/call.S'] +
                   [tmpdir+'/function.c'] +
                   self.sources + extra_sources)
        try:
            subprocess.check_output(cc_args)
        except subprocess.CalledProcessError:
            print("Command was:",' '.join(cc_args))
            raise

        oc_args = (['arm-none-eabi-objcopy'] +
                   ['-O','binary'] +
                   [tmpdir+'/a.out'] +
                   [tmpdir+'/stub.bin'])
        subprocess.check_output(oc_args)
        stub = open(tmpdir+'/stub.bin','rb').read()

        # If error, we leak a directory.  On purpose, for debugging
        shutil.rmtree(tmpdir)
        return stub

def example_nRF_stub(d, *args, **kwargs):
    """Example stub function.

    For editing, you want an editor that understands Python and C at
    the same time.

    Emacs has indirect buffers built in. (C-x 4 c) See also
    https://emacswiki.org/emacs/MultipleModes .  For other editors,
    you're on your own.

    This example assumes an nRF51 attached with LED on pin 12.

    This example is an example of using Python as a preprocessor,
    although it would probably be clearer to pass led_pin as an
    argument or #define.

    """
    if not 'led_pin' in kwargs:
        kwargs['led_pin']=12

    return d.make_stub(
        """
#include "nrf_gpio.h"
#include "nrf_delay.h"

void function(uint32_t r0) {

  int i;
  nrf_gpio_cfg_output(%(led_pin)d);
  for (i=0; i<r0; i++) {
    nrf_gpio_pin_toggle(%(led_pin)d);
    nrf_delay_ms(50);
  }
}
    """%kwargs,
        extra_include_path=[d.sdk+'/components/libraries/delay'],)
#extra_sources=[sdk+'/components/drivers_nrf/delay/nrf_delay.c'])

def example_bp_stub(d, *args, **kwargs):
    """Example stub function for Bluepill board.

    The only IO we have on this board is an LED on GPIOC.13, which
    turns on for output low.

    """

    return d.make_stub(
        """
#include <stdint.h>

static void delay(uint32_t ct) {
    int i;
    for (i=0; i<ct; i++) asm("nop");
}

#define LED_PORT GPIOC
#define LED_PIN (13)

void function(uint32_t r0) {

  int i;
  LED_PORT->CRH |= 0x1 << ((LED_PIN*4) %% 32);

  for (i=0; i<r0; i++) {
    LED_PORT->ODR ^= (1<<13);
    delay(30000);
  }
}
    """%kwargs)

def nrf51_example():
    from svd_gdb.drivers import nrf5x
    d = nrf5x.NRF51()

    stub = example_nRF_stub(d, led_pin=12)
    print(stub)
    d._parent.run_stub_timeout(30, stub, 0x20000000, 200)

def stm32_bp_example():
    from svd_gdb.drivers import stm32
    d = stm32.STM32F103()

    stub = example_bp_stub(d, led_pin=12)
    print(stub)
    d._parent.run_stub_timeout(30, stub, 0x20000000, 10)


if __name__=="__main__":
    stm32_bp_example()



#!/usr/bin/python3

from . import base
from .. import svd_gdb

class NRF5xPin(base.Pin):
    def __init__(self, parent, pinnumber):
        if pinnumber > 31:
            port = 1
            self.port = parent.P1
        else:
            port = 0
            self.port = parent.P0
        self.pinnummber = pinnumber
        self.pin = pinnumber & 31 # pin within port
        self.pinmask = 1<<self.pin

        self.name = "P%d_%d"%(port, self.pin)

    @property
    def i(self):
        "Reads digital input latch, in [0, 1]"
        self.port.DIRCLR = self.pinmask
        self.port.PIN_CNF[self.pin].INPUT = 0 # Not default!
        return int(bool(self.port.IN & self.pinmask))

    @property
    def o(self):
        "Reads digital output latch, in [0, 1]"
        return int(bool(self.port.OUT & self.pinmask))

    @o.setter
    def o(self, val):
        "Sets digital output latch to val"
        self.port.DIRSET = self.pinmask
        if (val):
            self.port.OUTSET = self.pinmask
        else:
            self.port.OUTCLR = self.pinmask

    @property
    def pull(self):
        "Returns selected pullup, in [None, 'h', 'l']"
        return [None, 'l', NotImplemented, 'h'][self.port.PIN_CNF[self.pin].PULL]

    @pull.setter
    def pull(self, v):
        "Sets selected pullup, in [None, 'h', 'l']"
        self.port.PIN_CNF[self.pin].PULL = {None: 0,
                                            'l': 1,
                                            'h': 3}[v]

    def __repr__(self):
        return self.name

class NRF5x(svd_gdb.Device):
    sdk = "/opt/nRF5_SDK_15.3.0/"
    mfgr_name = 'Nordic'
    svd_name = '' # override me

    def __init__(self, target=None):
        svd_filename = base.cmsis_svd_file(self.mfgr_name, self.svd_name)
        gdb_int = svd_gdb.GdbInterface(target)
        super().__init__(svd_filename, gdb_int)

        self._gdb.make_stub.include_path += [
            self.sdk+'/CMSIS/Include',
            self.sdk+'/modules/nrfx/mdk/',
            self.sdk+'/modules/nrfx/hal/',
            self.sdk+'/modules/nrfx/',
            self.sdk+'/integration/nrfx/',
            self.sdk+'/components/libraries/util/',
            self.sdk+'/components/drivers_nrf/nrf_soc_nosd/',
            self.sdk+'/components/toolchain/',
            self.sdk+'/components/toolchain/gcc',
            self.sdk+'/components/toolchain/cmsis/include'
        ]

    def _add_pins(self, pins=32):

        for pin_number in range(pins):
            pin = NRF5xPin(self, pin_number)
            setattr(self, pin.name, pin)
            self._pins.append(pin)

class NRF51(NRF5x):
    svd_name = 'nrf51.svd'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        assert 'nRF51' in self._gdb.target_name

        self._gdb.make_stub.include_path = [
            self.sdk+'/modules/nrfx/templates/nRF51/'
        ] + self._gdb.make_stub.include_path

        self._gdb.make_stub.includes += ['<nrf51.h>', '<nrf51_bitfields.h>']
        self._gdb.make_stub.defines += ['NRF51']

        self.P0 = self.GPIO # compatibility name
        self._add_pins(32)

class NRF52(NRF5x):
    svd_name = 'nrf52.svd'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        assert 'nRF52' in self._gdb.target_name

        self._gdb.make_stub.include_path = [
            self.sdk+'/modules/nrfx/templates/nrf52/'
        ] + self._gdb.make_stub.include_path

        self._gdb.make_stub.includes += ['<nrf52.h>', '<nrf52_bitfields.h>']
        self._gdb.make_stub.defines += ['NRF52']

        self._add_pins(32)

if __name__=="__main__":
    pass



#!/usr/bin/python3

from . import base
from .. import svd_gdb

class STM32F1Pin(base.Pin):
    def __init__(self, parent, port, pinnumber):
        self.parent = parent
        self.port = port
        self.pinnumber = pinnumber
        self.pin = pinnumber
        self.pinmask = 1<<self.pin

        self.name = "P%s_%d"%(port._name[-1:], self.pin)

    def c_macro(self, prefix):
        return """
#define %(prefix)sCNFMODE(val) \\
\t%(port)s->%(CR)s = ((%(port)s->%(CR)s & ~(0xf << (4u * (7 & %(pin)d)))) | \\
\t          ((val) << (4u * (7 & %(pin)d))))

#define %(prefix)sOUTHIGH do { \\
\t%(port)s->BSRR = (1u<<%(pin)du); \\
\t%(prefix)sCNFMODE(1); \\
} while (0)

#define %(prefix)sOUTLOW do { \\
\t%(port)s->BSRR = (0x10000u<<%(pin)du); \\
\t%(prefix)sCNFMODE(1); \\
} while (0)

#define %(prefix)sINPULLUP ( \\
\t%(prefix)sCNFMODE(8), \\
\t%(port)s->BSRR = (1u<<%(pin)du), \\
\t(!!(%(port)s->IDR & (1u<<%(pin)du))))

#define %(prefix)sIN ( \\
\t%(port)s->BSRR = (1u<<%(pin)du), \\
\t%(prefix)sCNFMODE(3), \\
\t(!!(%(port)s->IDR & (1u<<%(pin)du))))
        """%{'prefix':prefix,
             'CR':['CRL','CRH'][self.pin >= 8],
             'pin':self.pin,
             'port':self.port._name.split('.')[-1]}

    @property
    def i(self):
        "Reads digital input latch, in [0, 1]"
        self._setin()
        return int(bool(self.port.IDR & self.pinmask))

    @property
    def o(self):
        "Reads digital output latch, in [0, 1]"
        return int(bool(self.port.ODR & self.pinmask))

    def _setoutl(self, level):
        if (level):
            self.port.BSRR = self.pinmask
        else:
            self.port.BSRR = self.pinmask << 16

    @o.setter
    def o(self, val):
        "Sets digital output latch to val"
        #self.port.DIRSET = self.pinmask
        self._setoutl(val)
        self._setout()

    @property
    def pull(self):
        "Returns selected pullup, in [None, 'h', 'l']"
        if self._cnfmode != 0b1000:
            return None
        return 'lh'[self.o]

    def _setin(self):
        cnfmode = self._cnfmode
        cnfmode &= ~0b11
        self._cnfmode = cnfmode

    def _setout(self):
        "Sets to 10 MHz push-pull"
        self._cnfmode = 0b0001

    @property
    def _cnfmode(self):
        "Return CNF+MODE bits. 0..15"
        if self.pinnumber > 8:
            cr = self.port.CRH
        else:
            cr = self.port.CRL

        return 15 & (cr >> 4 * (7 & self.pinnumber))

    @_cnfmode.setter
    def _cnfmode(self, new_cnfmode):
        cr = self._cnfmode

        cr &= ~(0b1111 << 4 * (7 & self.pinnumber))
        cr |= (new_cnfmode << 4 * (7 & self.pinnumber))

        if self.pinnumber > 8:
            self.port.CRH = cr
        else:
            self.port.CRL = cr

    @pull.setter
    def pull(self, v):
        "Sets selected pullup, in [None, 'h', 'l']"
        cnfmode,out = {None:(0b1001,0),
                       'l':(0b1000,0),
                       'h':(0b1000,1)}[v]
        # for input, MODE0 = 00
        # for pull-up input, set CNF1, clear CNF0
        # ODR sets pull dirction
        self._cnfmode = cnfmode
        if cnfmode:
            self._setoutl(out)

def STM32F3Pin(parent, port, pinnumber):
    pinname = 'P%s%d'%(port._name[-1:], pinnumber)

    analog_map = {'PC0':6,
                  'PC1':7,
                  'PC2':8,
                  'PC3':9,
                  'PA0':1,
                  'PA1':2,
                  'PA2':3,
                  'PA3':4,
                  'PA4':5,
                  'PA6':10,
                  'PA7':15,
                  'PB0':11,
                  'PB1':12,
                  'PB11':14,
                  'PB13':13}
    if pinname in analog_map:
        return STM32F3AnalogPin(parent, port, pinnumber, analog_map[pinname])
    else:
        return STM32F3DigitalPin(parent, port, pinnumber)

class STM32F3DigitalPin(base.Pin):
    def __init__(self, parent, port, pinnumber):
        self.parent = parent
        self.port = port
        self.pinnumber = pinnumber
        self.pin = pinnumber
        self.pinmask = 1<<self.pin

        self.name = "P%s_%d"%(port._name[-1:], self.pin)

    @property
    def i(self):
        "Reads digital input latch, in [0, 1]"
        self._setin()
        return int(bool(self.port.IDR & self.pinmask))

    @property
    def o(self):
        "Reads digital output latch, in [0, 1]"
        return int(bool(self.port.ODR & self.pinmask))

    def _setoutl(self, level):
        if (level):
            self.port.BSRR = self.pinmask
        else:
            self.port.BSRR = self.pinmask << 16

    @o.setter
    def o(self, val):
        "Sets digital output latch to val"
        #self.port.DIRSET = self.pinmask
        self._setoutl(val)
        self._setout()

    def _set_moder(self, new_mode):
        moder = self.port.MODER
        moder &= ~(0b11 << (2*self.pinnumber))
        moder |= new_mode << (2*self.pinnumber)
        self.port.MODER=moder

    def _setin(self):
        self._set_moder(0)

    def _setout(self):
        self._set_moder(1)

    @property
    def pull(self):
        "Returns selected pullup, in [None, 'h', 'l']"
        pupdr = 3 & self.port.PUPDR >> (2*self.pinnumber)
        return {0b00:None,
                0b01:'h',
                0b10:'l',
                0b11:NotImplemented}[pupdr]

    @pull.setter
    def pull(self, v):
        "Sets selected pullup, in [None, 'h', 'l']"
        pupdr = {None:0b00,
                 'h':0b01,
                 'l':0b10}[v]

        reg = self.port.PUPDR & ~(3 << (2*self.pinnumber))
        reg |= pupdr << (2*self.pinnumber)
        self.port.PUPDR = reg

class STM32F3AnalogPin(STM32F3DigitalPin):
    def __init__(self, parent, port, pinnumber, analog_ch):
        super().__init__(parent,port,pinnumber)
        self.analog_ch = analog_ch

    @property
    def v(self):
        return 3.3*self.rawadc/4096

    @property
    def rawadc(self):
        self.parent._init_adc()
        adc = self.parent.ADC
        self._set_moder(3)

        adc.SQR1 = self.analog_ch << adc.SQR1.SQ1._bit_offset
        adc.CR.ADSTART = 1
        # USB delay is enough
        return adc.DR

class STM32F(svd_gdb.Device):
    sdk = "/opt/cubeMX/Drivers"
    mfgr_name = 'STMicro'
    svd_name = ''

    def __init__(self, target=None):
        svd_filename = base.cmsis_svd_file(self.mfgr_name, self.svd_name)
        #svd_filename = self.svd_name
        gdb_int = svd_gdb.GdbInterface(target)
        super().__init__(svd_filename, gdb_int)
        self._gdb.make_stub.include_path.append(self.sdk+'/CMSIS/Include')

    def _ports_on(self, reg):
        # power up all the GPIOs by building a mask for RCC
        enbits = [b
                  for b in dir(reg)
                  if b.startswith('IOP') and b.endswith('EN')]
        mask = 0
        for bit in enbits:
            field = getattr(reg, bit)
            mask |= 1<<field._bit_offset

        reg |= mask

class STM32F1(STM32F):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        assert 'STM32F1' in self._gdb.target_name

        for gpio in (x for x in dir(self) if x.startswith('GPIO')):
            port = getattr(self, gpio)
            self._add_pins(port, 16)

        self._ports_on(self.RCC.APB2ENR)

        self._gdb.make_stub.include_path.append(self.sdk+'/CMSIS/Device/ST/STM32F1xx/Include')
        self._gdb.make_stub.cflags.append('-mcpu=cortex-m3')

    def _add_pins(self, port, pins=8):

        for pin_number in range(pins):
            pin = STM32F1Pin(self, port, pin_number)
            setattr(self, pin.name, pin)
            self._pins.append(pin)

class STM32F3(STM32F):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        assert 'STM32F3' in self._gdb.target_name

        for gpio in (x for x in dir(self) if x.startswith('GPIO')):
            port = getattr(self, gpio)
            self._add_pins(port, 16)

        self._ports_on(self.RCC.AHBENR)
        self._gdb.make_stub.cflags.append('-mcpu=cortex-m4')
        self._gdb.make_stub.include_path.append(self.sdk+'/CMSIS/Device/ST/STM32F3xx/Include')
        self._adc_initted = False

    def _add_pins(self, port, pins=8):

        for pin_number in range(pins):
            pin = STM32F3Pin(self, port, pin_number)
            setattr(self, pin.name, pin)
            self._pins.append(pin)

    def _init_adc(self):
        if not self._adc_initted:
            # Have to assume no adc is set up.
            self.RCC.AHBRSTR.ADC1RST = 1
            self.RCC.AHBRSTR.ADC1RST = 0
            self.RCC.CFGR2.ADC1PRES = 0
            self.RCC.AHBENR.ADC1EN = 1
            adc = self.ADC
            adc.CRR.CKMODE = 1 # async
            adc.CR = 0 # All off, regulator prep
            adc.CR.ADVREGEN = 1 # regulator on

            adc.CR.ADCALDIF = 0
            adc.CR.ADCAL = 1
            while (adc.CR.ADCAL):
                pass

            adc.CR.ADEN = 1
            adc.CFGR = 0
            adc.CFGR.OVRMOD = 1 # enable overrun
            adc.SMPR1 = 0
            adc.SMPR1.SMP1 = 5 # 61.5 ADC cycles
            adc.DIFSEL = 0
            self._adc_initted = True

class STM32F103(STM32F1):
    svd_name = 'STM32F103xx.svd'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._gdb.make_stub.includes += ['"stm32f103xb.h"']
        assert 'STM32F1' in self._gdb.target_name

class STM32F301(STM32F3):
    svd_name = 'STM32F301.svd'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._gdb.make_stub.includes += '"stm32f301x8.h"'
        assert 'STM32F3' in self._gdb.target_name

if __name__=="__main__":
    pass



#!/usr/bin/python3

import numbers
import math

class MutableNumber(numbers.Integral):

    """Emulate numeric types based on "_n" attribute, based on
    https://docs.python.org/3/reference/datamodel.html#basic-customization
    https://docs.python.org/3/reference/datamodel.html#emulating-numeric-types

    This is effectively a mutable container for a number, and can be
    subclassed to provide interesting properties that are related to
    the number.

    To use this class, subclass (if you like) or construct it
    directly, passing the numerical object you wish to make mutable to
    the initializer.

    The current immutable number is the '_n' attribute, and all the
    numeric type dunder methods just delegate to this current number.

    """

    # Basic customization
    def __init__(self, n=None): self._n=n
    def __repr__(self): return repr(self._n)
    def __str__(self): return str(self._n)
    # def __bytes__(self): unimplemented
    def __format__(self, format_spec):
        return self._n.__format__(format_spec)

    def __index__(self): return self._n.__index__()

    def __lt__(self, other): return self._n < other
    def __le__(self, other): return self._n <= other
    def __ne__(self, other): return self._n != other
    def __eq__(self, other): return self._n == other
    def __gt__(self, other): return self._n > other
    def __ge__(self, other): return self._n >= other

    def __hash__(self): return hash(self._n)
    def __bool__(self): return bool(self._n)

    # math functions
    def __ceil__(self): return math.ceil(self._n)
    def __floor__(self): return math.floor(self._n)
    def __trunc__(self): return math.trunc(self._n)

    # Binary arithmetic operations
    def __add__(self, other): return self._n + other
    def __sub__(self, other): return self._n - other
    def __mul__(self, other): return self._n * other
    def __truediv__(self, other): return self._n / other
    def __floordiv__(self, other): return self._n // other
    def __mod__(self, other): return self._n % other
    def __divmod__(self, other): return divmod(self._n, other)
    def __pow__(self, other, modulo=None): return pow(self._n, other, modulo)

    # Right binary operations
    def __radd__(self, other): return other + self._n
    def __rsub__(self, other): return other - self._n
    def __rmul__(self, other): return other * self._n
    def __rtruediv__(self, other): return other / self._n
    def __rfloordiv__(self, other): return other // self._n
    def __rmod__(self, other): return other % self._n
    def __rdivmod__(self, other): return divmod(other, self._n)
    def __rpow__(self, other): return pow(other, self._n)

    # In-place binary operations
    def __iadd__(self, other):
        self._n += other
        return self
    def __isub__(self, other):
        self._n -= other
        return self
    def __imul__(self, other):
        self._n *= other
        return self
    def __itruediv__(self, other):
        self._n /= other
        return self
    def __ifloordiv__(self, other):
        self._n //= other
        return self
    def __imod__(self, other):
        self._n %= other
        return self
    def __ipow__(self, other, modulo=None):
        self._n = pow(self._n, other, modulo)
        return self

    # Unary arithmetic operations
    def __neg__(self): return -self._n
    def __pos__(self): return +self._n
    def __abs__(self): return abs(self._n)

    # Conversion functions
    def __complex__(self): return complex(self._n)
    def __int__(self): return int(self._n)
    def __float__(self): return float(self._n)
    def __round__(self, n=0): return round(self._n, n)

class MutableInteger(MutableNumber):

    def __lshift__(self, other): return self._n << other
    def __rshift__(self, other): return self._n >> other
    def __and__(self, other): return self._n & other
    def __xor__(self, other): return self._n ^ other
    def __or__(self, other): return self._n | other

    def __rlshift__(self, other): return other << self._n
    def __rrshift__(self, other): return other >> self._n
    def __rand__(self, other): return other & self._n
    def __rxor__(self, other): return other ^ self._n
    def __ror__(self, other): return other | self._n

    def __ilshift__(self, other):
        self._n <<= other
        return self
    def __irshift__(self, other):
        self._n >>= other
        return self
    def __iand__(self, other):
        self._n &= other
        return self
    def __ixor__(self, other):
        self._n ^= other
        return self
    def __ior__(self, other):
        self._n |= other
        return self

    def __invert__(self): return ~self._n

    # integer functions
    # https://docs.python.org/3/library/stdtypes.html#additional-methods-on-integer-types
    def bit_length(self): return self._n.bit_length()
    def to_bytes(self, length, byteorder, *args, signed=False):
        return self._n.to_bytes(length, byteorder, *args, signed=signed)
    def from_bytes(self, bytes, byteorder, *args, signed=False):
        return self._n.from_bytes(bytes, byteorder, *args, signed=signed)

    def hex(self): return self._n.hex()

class MutableFloat(MutableNumber):
    """ Add some float operations """

    def conjugate(self): return self._n.conjugate()

    @property
    def denominator(self): return self._n.denominator
    @property
    def numerator(self): return self._n.numerator

    @property
    def imag(self): return self._n.imag
    @property
    def real(self): return self._n.real

    # float functions
    # https://docs.python.org/3/library/stdtypes.html#additional-methods-on-float
    def as_integer_ratio(self): return self._n.as_integer_ratio()
    def is_integer(self): return self._n.is_integer()



#!/usr/bin/python3

import xml.etree.ElementTree as ET
import textwrap

def int0(s):
    s = s.lower()
    if s.startswith('0x'):
        return int(s,16)
    else:
        return int(s)

def bool0(s):
    s = s.lower().strip()
    return s in ['1','true']

from .mutable_number import MutableInteger
class Int32(int):
    def __str__(self):
        return "0x%08x"%self

    def __repr__(self):
        return "0x%08x"%self

class IntX(int):
    def __new__(cls, width, *args, **kwargs):
        if 'extra' in kwargs:
            extra = kwargs['extra']
            del kwargs['extra']
        else:
            extra = None
        obj = int.__new__(cls, *args, **kwargs)
        obj._width = width
        obj._extra = extra
        return obj

    def __str__(self):
        w = 1+(self._width-1)//4
        return "0x%%0%dx"%w%self

    def __repr__(self):
        ret = str(self)
        if self._extra:
            ret += ' '+self._extra
        return ret

class EnumeratedValue():
    def __init__(self, value, name, description):
        "value is a string type, decoded into value and mask according to enumeratedValue rules"
        if type(value)==str:
            value = value.lower()
            binvalue = None
            if value.startswith('0b'):
                binvalue = value[2:]
            elif value.startswith('#'):
                binvalue = value[1:]

            if binvalue is None:
                self._values = set((int0(value),))
            else:
                def both(x):
                    ret = []
                    for y in x:
                        if not 'x' in y:
                            ret.append(y)
                        else:
                            ret += both([y.replace('x','0',1)])
                            ret += both([y.replace('x','1',1)])
                    return ret
                self._values = set(int(x,2) for x in both([binvalue]))
        else:
            self._values = set(value)

        self._name = name
        self._description = description

    def __eq__(self, other):
        return other in self._values
    def __int__(self):
        return self._values[0]

class SvdObj():
    _strfields = (('_name','name'),
                  ('_description','description'))
    _intfields = ()

    def __init__(self, svd, parent):
        self._svd = svd
        self._parent = parent
        self._gdb = parent._gdb

        for attname, fieldname in self._strfields:
            x = svd.find(fieldname)
            if x is not None:
                x = x.text
                if x is not None: # Yep, Element.text returns None for zero-length string.
                    x = x.strip()
                    x = ' '.join(x.split()) # Remove weird whitespace
            setattr(self, attname, x)

        for attname, fieldname in self._intfields:
            x = svd.find(fieldname)
            if x is not None:
                x = int0(x.text)
            setattr(self, attname, x)

    def __repr__(self):
        return repr(self._parent)+'.'+self._name

class CPU(SvdObj):
    def __init__(self, svd, parent):
        self._svd = svd
        self._name = svd.find('name').text.strip()
    def __repr__(self):
        return self._name

class ElementGroup(tuple):
    """ Read-only list of registers, but you can assign values
    to individual registers. """

    def __setitem__(self, key, value):
        if isinstance(key, slice):
            indexes = range(*key.indices(len(self)))

            if len(indexes) != len(value):
                raise ValueError()

            for i,v in zip(indexes, value):
                self[i] = v

        else:
            while key < 0:
                key += len(self)
            self[key]._set(value)

    def __getitem__(self, key):
        ret = super().__getitem__(key)

        if isinstance(key, slice):
            return self.__class__(ret)
        else:
            return ret

    def __repr__(self):
        return repr(list(self))

    @property
    def _child_regs(self):
        ret = []
        for s in self:
            ret.extend(s._child_regs)

        return ret

class Field(SvdObj, MutableInteger):
    def __init__(self, svd, parent):
        super().__init__(svd, parent)

        self._bit_offset = 0
        self._bit_width = 32

        b1 = svd.find('bitOffset')
        b2 = svd.find('lsb')
        b3 = svd.find('bitRange')
        if b1 is not None:
            self._bit_offset = int0(b1.text)
            self._bit_width = int0(svd.find('bitWidth').text)
        elif b2 is not None:
            self._bit_offset = int0(b2.text)
            self._bit_width = 1 + int0(svd.find('msb').text) - self._bit_offset
        elif b3 is not None:
            'A string in the format: "[<msb>:<lsb>]"'
            s = b3.text.strip()
            assert s[0]=='['
            assert s[-1]==']'
            msb,lsb = s[1:-1].split(':')
            self._bit_offset = int0(lsb)
            self._bit_width = 1 + int0(msb) - self._bit_offset
        else:
            assert False
        assert self._bit_width >= 1

        self._enum = []
        ev = svd.find('enumeratedValues')
        if ev is not None:
            default_value = None
            for val in ev.findall('enumeratedValue'):
                name = val.find('name').text.strip()
                d = val.find('description')
                if d is not None:
                    d = d.text
                    if d is not None:
                        d = d.strip()
                description = d

                value = val.find('value')
                if value is None:
                    isd = val.find('isDefault')
                    if isd is not None and bool0(isd.text):
                        default_value = (name, description)
                else:
                    value = value.text.strip()
                    self._enum.append(EnumeratedValue(value,name,description))
            if default_value:
                all_values = list(range(1<<self._bit_width))
                for v in all_values:
                    if v in self._enum:
                        all_values.remove(v)
                self._enum.append(EnumeratedValue(all_values,name,description))

    def _set(self, value):
        if self._bit_width==1:
            if value & 1:
                self._parent._set_bit(self._bit_offset)
            else:
                self._parent._clear_bit(self._bit_offset)
        elif self._bit_width==32:
            self._parent_set(value)
        else:
            mask = ((1 << self._bit_width)-1) << self._bit_offset
            x = self._parent._get()
            x &= ~mask
            x |= value << self._bit_offset
            self._parent._set(x)

    def _get(self):
        if self._bit_width==1:
            ret = self._parent._is_bit_set(self._bit_offset)
        elif self._bit_width==32:
            ret = self._parent._get()
        else:
            mask = ((1 << self._bit_width)-1)
            x = self._parent._get()
            x >>= self._bit_offset
            ret = x & mask

        try:
            i = self._enum.index(ret)
        except ValueError:
            return IntX(self._bit_width, ret)

        extra = "(%s)"%self._enum[i]._name
        return IntX(self._bit_width, ret, extra=extra)

    @property
    def _n(self):
        return self._get()

    @_n.setter
    def _n(self, v):
        return self._set(v)

    def __repr__(self):
        return self._parent._repr_no_get()+'.'+self._name+' = '+repr(self._get())

class FieldHaver():
    def __setattr__(self, attname, value):
        if hasattr(self, '_freezeattr'): # avoid recursion
            if attname != '_freezeattr':
                if attname == '_n':
                    self._set(value)
                else:
                    self._fieldmap[attname]._set(value)
        else:
            super().__setattr__(attname, value)

class Register(SvdObj, FieldHaver, MutableInteger):
    _intfields = (('_address_offset','addressOffset'),)

    def __init__(self, svd, parent):
        super().__init__(svd, parent)

        self._fields = []
        self._fieldmap = {}
        fsvd = svd.find('fields')

        if fsvd is not None:
            for f in fsvd.findall('field'):
                # Deal with derived fields when we actually find an
                # SVD with derived field.

                xx = f.attrib.get('derivedFrom',None)
                assert xx is None
                for ff in make_field(f, self):
                    self._fields.append(ff)
                    setattr(self, ff._name, ff)
                    self._fieldmap[ff._name] = ff

    @property
    def _address(self):
        return Int32(self._address_offset + self._parent._address)

    def _get(self):
        return Int32(self._gdb.read32(self._address))

    def _set(self, value):
        return self._gdb.write32(self._address, int(value))

    def _set_bit(self, bit):
        return self._gdb.set_bit(self._address, bit)

    def _clear_bit(self, bit):
        return self._gdb.clear_bit(self._address, bit)

    def _is_bit_set(self, bit):
        return self._gdb.is_bit_set(self._address, bit)

    def _repr_no_get(self):
        return super().__repr__()

    def __repr__(self):
        return self._repr_no_get()+' = '+repr(self._get())

    @property
    def _n(self):
        return self._get()

    @property
    def _child_regs(self):
        return [self]

    def _dump_repr(self):
        pad = ' '*12
        ret = pad+self._repr_no_get()+'\n'
        if self._description:
            for descline in textwrap.wrap(self._description.strip(),60):
                ret += pad+'| '+descline+'\n'
        v = self._get()
        vs = ''.join('   '+x for x in "%08x"%v)
        ret += "%s: 0x%s\n"%(self._address, vs)
        fs = self._fields[:]
        fs.sort(key=lambda x:x._bit_offset)
        fmaplines = []
        binary = format(v, '032b')
        fmaplines.append(binary)
        if fs:
            underlines = [' ']*len(binary)
            for f in fs:
                underlines[32-f._bit_offset-f._bit_width:32-f._bit_offset] = (
                    ['^'] * f._bit_width)
            fmaplines.append(''.join(underlines))

            for f_ind in range(len(fs)):
                f0 = fs[f_ind]
                frem = fs[f_ind+1:]

                nextline = [' ']*len(binary)
                nextline[32-f0._bit_offset-1:] = ['+']+(['-']*f0._bit_offset)
                for f in frem:
                    nextline[32-f._bit_offset-1] = '|'
                desc = f0._description or ''
                tmp = '%s = %r'%(f0._name,f0._get())
                fmaplines.append('%s %14s | %s'%(''.join(nextline),tmp,desc))
        fmaplines[0] = '0b'+fmaplines[0]
        fmaplines[1:] = ['  '+f for f in fmaplines[1:]]

        for fmapline in fmaplines:
            ret += pad+fmapline+'\n'

        ret += '\n'
        return ret

    def _dump(self):
        print(self._dump_repr())

class RegHaver():
    def __setattr__(self, attname, value):
        if hasattr(self, '_freezeattr'): # avoid recursion
            self._registermap[attname]._set(value)
        else:
            super().__setattr__(attname, value)

def regs_and_clusters(self, svd):

    self._registers = getattr(self, '_registers', [])
    self._registermap = getattr(self, '_registermap', {})
    self._clusters = getattr(self, '_clusters', [])

    if svd is None:
        return

    for r in svd.findall('register'):
        for rr in make_register(r, self):
            self._registers.append(rr)
            self._registermap[rr._name] = rr
            setattr(self, rr._name, rr)
            rr._freezeattr = True

    for c in svd.findall('cluster'):
        for cc in make_cluster(c, self):
            self._clusters.append(cc)
            setattr(self, cc._name, cc)

class Cluster(SvdObj, RegHaver):
    _intfields = (('_address_offset','addressOffset'),)

    def __init__(self, svd, parent):
        super().__init__(svd, parent)

        regs_and_clusters(self, svd)

    def __repr__(self):
        return repr(self._parent)+'.'+self._name

    @property
    def _address(self):
        return Int32(self._address_offset + self._parent._address)

    @property
    def _child_regs(self):
        ret = []
        for c in self._registers+self._clusters:
            ret.extend(c._child_regs)

        return ret

def fixup_eg_name(n):
    "asdf[%s] -> asdf, False otherwise"
    if n.endswith('[%s]'):
        return n[:-4]
    else:
        return False

def make_cls_or_array(svd, parent, cls):
    """If there is no dim, just create a new object and return it.
    If there is a plain dim like NAME[%s] then we need to return array-like.
    If there is a dimIndex defined, then we need to return a list.

    For convenience, we will *always* return a list, and the caller
    won't need special-case code to apply it.

    """

    dim = svd.find('dim')
    if dim is None:
        return [cls(svd, parent)]
    else:
        dim = int0(dim.text)
        dimIncrement = int0(svd.find('dimIncrement').text)

        di = svd.find('dimIndex')
        if di is not None:
            t = di.text.strip()
            if "," in t:
                dimIndex = t.split(',')
            elif "-" in t:
                first,last = t.split("-")
                dimIndex = [str(x) for x in range(int(first),1+int(last))]
        else:
            dimIndex = [str(x) for x in range(dim)]

        assert len(dimIndex) == dim

        objs = []
        for r in range(dim):
            obj = cls(svd, parent)
            if cls==Field:
                obj._bit_offset += r*dimIncrement
            else:
                obj._address_offset += r*dimIncrement
            obj._name %= dimIndex[r]
            obj._freezeattr = True
            objs.append(obj)

        trimmed_name = fixup_eg_name(svd.find('name').text)
        if trimmed_name:
            eg = ElementGroup(objs)
            eg._name = trimmed_name
            return [eg]
        else:
            return objs

def make_field(svd, parent):
    return make_cls_or_array(svd, parent, Field)

def make_cluster(svd, parent):
    return make_cls_or_array(svd, parent, Cluster)

def make_register(svd, parent):
    return make_cls_or_array(svd, parent, Register)

class Peripheral(RegHaver):
    def __init__(self, svd, parent):
        self._parent = parent
        self._gdb = parent._gdb
        self._update(svd)

    def _update(self, svd):
        self._svd = svd

        self._name = svd.findtext('name').strip()
        d = svd.find('description')
        if d is not None:
            self._description = d.text

        self._address = Int32(int0(svd.find('baseAddress').text))

        rsvd = svd.find('registers')

        regs_and_clusters(self, rsvd)

    def _copy(self):
        c = self.__class__(self._svd, self._parent)
        try:
            del c._freezeattr
        except AttributeError:
            pass
        assert c._registermap.keys() == self._registermap.keys()
        return c

    def __repr__(self):
        return repr(self._parent)+'.'+self._name

    @property
    def _child_regs(self):
        regs = []
        for c in self._registers + self._clusters:
            regs.extend(c._child_regs)

        regs.sort(key=lambda x:x._address)
        return regs

    def _dump(self, file=None):
        """ Pretty-printed look at all the registers in a peripheral """

        dump = '\n'.join([r._dump_repr() for r in self._child_regs])
        if file:
            file.write(dump)
        else:
            import pydoc
            pydoc.getpager()(dump)

class DebugInterface():
    """ Functions to interface from the device description to the gdb proxy object.
    Dummy object, subclass for more interesting object """
    verbose=False

    def setup_make_stub(self, svd_device):
        pass

    def read32(self, address):
        if self.verbose:
            print("Read from",repr(address))
        return Int32(0)

    def write32(self, address, val):
        if self.verbose:
            print("Write",val,"to",repr(address))
        pass

    def _set_bit_rmw(self, address, bit):
        x = self.read32(address)
        x |= (1<<bit)
        self.write32(address, x)

    def _clear_bit_rmw(self, address, bit):
        x = self.read32(address)
        x &= ~(1<<bit)
        self.write32(address, x)

    def set_bit(self, address, bit):
        return self._set_bit_rmw(address, bit)

    def clear_bit(self, address, bit):
        return self._clear_bit_rmw(address, bit)

    def is_bit_set(self, address, bit):
        return bool(self.read32(address) & (1<<bit))

    @property
    def target_name(self):
        return self.gdb.target_name

from . import gdb

def get_first_swd():
    import serial

    s=serial.Serial('/dev/ttyBmpGdb',timeout=0.3)
    target = gdb.Target(s)

    target.monitor('swdp_scan')
    target.attach(1)
    return target


class GdbInterface(DebugInterface):
    def __init__(self, gdb_):
        if gdb_ is None:
            gdb_ = get_first_swd()

        self.gdb = gdb_

    def setup_make_stub(self, svd_device):
        from . import make_stub
        self.make_stub = make_stub.MakeStub(svd_device)

    def read32(self, address):
        val = self.gdb.read32(address)
        if self.verbose:
            print("Read from",repr(address),"= %08x"%val)
        return val

    def write32(self, address, val):
        if self.verbose:
            print("Write %08x"%val,"to",repr(address))
        return self.gdb.write32(address, val)

    @property
    def target_name(self):
        return self.gdb.target_name

class Device():
    def __init__(self, xmlfilename, gdb=None):
        if gdb is None:
            gdb = DebugInterface()

        self._gdb = gdb

        # print(xmlfilename.split('/')[-1])
        tree = ET.parse(xmlfilename)
        root = tree.getroot()

        self._name = root.find('name').text.strip()

        c = root.find('cpu')
        if c is not None:
            self._cpu = CPU(root.find('cpu'), self)
        else:
            self._cpu = None

        svd = root.find('peripherals')

        self._peripherals = []
        for psvd in svd.findall('peripheral'):
            df = psvd.attrib.get('derivedFrom',None)
            if df:
                p = getattr(self, df)._copy()
                p._update(psvd)
            else:
                p = Peripheral(psvd, self)
            p._freezeattr = True
            setattr(self, p._name, p)
            self._peripherals.append(p)

        self._r=root
        self._gdb.setup_make_stub(self)

        self._pins = []

    def __repr__(self):
        return self._name

#d = Device('nrf52.svd')

import sys
for arg in sys.argv[1:]:
    d = Device(arg)

    try:
        print(repr(d.TIM1)+'.OR at', d.TIM1.OR._address)
    except AttributeError:
        pass

    #del d



#!/usr/bin/python3

class Pin():
    "ABC for microprocessor pins"

    @property
    def v(self):
        "Return voltage (for ADC-capable pins). Floating-point."
        return NotImplemented

    @property
    def i(self):
        "Reads digital input, in [0, 1]"
        return NotImplemented

    @property
    def o(self):
        "Reads digital output latch, in [0, 1]"
        return NotImplemented

    @o.setter
    def o(self, val):
        "Sets digital output latch to val"
        raise Exception(NotImplemented)

    @property
    def pull(self):
        "Returns selected pullup, in [None, 'h', 'l']"
        return NotImplemented

    @pull.setter
    def pull(self, updown):
        "Sets pullup/down, one of [None, 'h', 'l'].  Sets to input"
        raise Exception(NotImplemented)

    @property
    def hiz(self):
        """For digital pin, uses pullups to guess if it's connected to a
        high-impedance"""
        d = []
        psave = self.pull
        self.pull = 'h'; d.append(self.i)
        self.pull = 'l'; d.append(self.i)
        self.pull = psave
        return d == [1, 0]

class Accel():
    name = "unknwon accelerometer"

    def read(self):
        """ Returns scaled acceleration, raw temperature """
        raise Exception('subclass me')

def cmsis_svd_file(vendor, filename):
    import cmsis_svd.parser

    rf = cmsis_svd.parser.pkg_resources.resource_filename
    return rf("cmsis_svd", "/".join(['data',vendor,filename]))



#!/usr/bin/python3

from setuptools import setup, find_packages

setup(name='svd_gdb',
      version='0.0.3',
      description='Python interface to SVD files through GDB',
      url='http://github.com/markrages/svd_gdb',
      author='Mark Rages',
      author_email='markrages@gmail.com',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
          'Programming Language :: Python :: 3.6',
          'Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator',
      ],
      packages=['svd_gdb', 'svd_gdb.drivers'],
      install_requires=['pySerial', 'cmsis_svd'],
      py_modules=['gdb'],
      zip_safe=False)
